@@ -10,14 +10,12 @@ def default_cache_func(
         django_model,
         cache_conf,
 ):
-    redis_dicts = redis_root.get(django_model, return_dict=True)
-    cache_to_django(
+    django_to_cache(
         redis_root,
         django_model,
-        redis_dicts,
         cache_conf,
     )
-    django_to_cache(
+    cache_to_django(
         redis_root,
         django_model,
         cache_conf,
@@ -27,15 +25,14 @@ def default_cache_func(
 def cache_to_django(
         redis_root,
         django_model,
-        redis_dicts,
         cache_conf,
 ):
     async def run_cache_to_django(
             redis_root,
             django_model,
-            redis_dicts,
             cache_conf,
     ):
+        redis_dicts = redis_root.get(django_model, return_dict=True)
         async_chunk_size = redis_root.async_db_requests_limit
         async_tasks = []
         completed = 0
@@ -46,6 +43,7 @@ def cache_to_django(
                 update_or_create_django_instance_from_redis_instance(
                     redis_instance_data,
                     django_model,
+                    cache_conf,
                 )
             )
             completed += 1
@@ -53,13 +51,14 @@ def cache_to_django(
                 await asyncio.gather(*async_tasks)
                 async_tasks = []
                 print(
-                    f'{datetime.datetime.now()} - Written {completed} {django_model.__name__} instances from cache to django')
+                    f'{datetime.datetime.now()} - '
+                    f'Written {completed} {django_model} instances from cache to django')
 
         if cache_conf['delete']:
 
             async def check_if_need_to_delete_django_instance(redis_instances_django_ids, django_instance):
                 if django_instance.id not in redis_instances_django_ids:
-                    django_instance.delete()
+                    await django_sync_to_async_list(django_instance.delete)
 
             all_redis_instances = redis_root.get(django_model)
             redis_instances_django_ids = []
@@ -67,7 +66,7 @@ def cache_to_django(
                 if 'django_id' in redis_instance.keys():
                     redis_instances_django_ids.append(redis_instance['django_id'])
 
-            all_django_instances = await django_sync_to_async_list(django_model.objects.all())
+            all_django_instances = await django_sync_to_async_list(django_model.objects.all)
             async_tasks = []
             completed = 0
             to_complete = len(all_django_instances)
@@ -82,7 +81,8 @@ def cache_to_django(
                 if len(async_tasks) == async_chunk_size or completed == to_complete:
                     await asyncio.gather(*async_tasks)
                     async_tasks = []
-                    print(f'{datetime.datetime.now()} - Deleted {django_model.__name__} instances from django')
+                    print(f'{datetime.datetime.now()} - '
+                          f'Deleted {django_model} instances from django')
 
             await asyncio.gather(*async_tasks)
 
@@ -90,7 +90,6 @@ def cache_to_django(
     loop.run_until_complete(run_cache_to_django(
         redis_root,
         django_model,
-        redis_dicts,
         cache_conf,
     ))
 
@@ -98,11 +97,13 @@ def cache_to_django(
 async def update_or_create_django_instance_from_redis_instance(
         redis_instance,
         django_model,
+        cache_conf,
 ):
     django_id = redis_instance['django_id']
     django_params, django_many_to_many_params = await async_redis_dict_to_django_params(
         redis_instance,
         django_model,
+        cache_conf,
     )
     new_django_params = {}
     for k, v in django_params.items():
@@ -114,6 +115,7 @@ async def update_or_create_django_instance_from_redis_instance(
         django_model,
         django_params,
         django_many_to_many_params,
+        cache_conf,
     )
 
     return django_instance
@@ -125,6 +127,7 @@ def sync_to_async_update_or_create_django_instance_from_redis_instance(
         django_model,
         django_params,
         django_many_to_many_params,
+        cache_conf,
 ):
     need_to_create = True
     if django_id not in ['null', None]:
@@ -134,14 +137,22 @@ def sync_to_async_update_or_create_django_instance_from_redis_instance(
     django_instance = None
 
     if need_to_create:
-        pass
-        print(need_to_create, django_params, django_many_to_many_params)
-    #     django_instance = django_model.objects.create(**django_params)
-    #     update_django_many_to_many(
-    #         django_model,
-    #         django_instance.id,
-    #         django_many_to_many_params
-    #     )
+        if cache_conf['write_to_django']:
+            django_instance = django_model.objects.create(**django_params)
+            update_django_many_to_many(
+                django_model,
+                django_instance.id,
+                django_many_to_many_params
+            )
+        else:
+            print(f'\n'
+                  f'{datetime.datetime.now()} - '
+                  f'Setting write_to_django is turned off, need to write:\n'
+                  f'Create {django_model} with params: \n'
+                  f'{django_params}\n'
+                  f'and many to many params\n'
+                  f'{django_many_to_many_params}'
+                  f'\n')
     else:
         django_instance = django_model.objects.filter(id=django_id)[0]
         changed_fields_to_update = {}
@@ -162,20 +173,30 @@ def sync_to_async_update_or_create_django_instance_from_redis_instance(
             django_instance_fields_value = list(getattr(django_instance, redis_field_name).all())
             if django_instance_fields_value != redis_field_value:
                 changed_many_to_many_fields_to_update[redis_field_name] = redis_field_value
-        print(need_to_create, changed_fields_to_update, changed_many_to_many_fields_to_update)
-    #     django_model.objects.filter(id=django_id).update(**changed_fields_to_update)
-    #     update_django_many_to_many(
-    #         django_model,
-    #         django_id,
-    #         changed_many_to_many_fields_to_update
-    #     )
-    #     django_instance = django_model.objects.filter(id=django_id)[0]
+        if cache_conf['write_to_django']:
+            django_model.objects.filter(id=django_id).update(**changed_fields_to_update)
+            update_django_many_to_many(
+                django_model,
+                django_id,
+                changed_many_to_many_fields_to_update
+            )
+            django_instance = django_model.objects.filter(id=django_id)[0]
+        else:
+            print(f'\n'
+                  f'{datetime.datetime.now()} - '
+                  f'Setting write_to_django is turned off, need to write:\n'
+                  f'Update {django_model} (ID {django_id}) with params: \n'
+                  f'{changed_fields_to_update}\n'
+                  f'and many to many params\n'
+                  f'{changed_many_to_many_fields_to_update}'
+                  f'\n')
     return django_instance
 
 
 async def async_redis_dict_to_django_params(
         redis_dict,
-        django_model
+        django_model,
+        cache_conf,
 ):
     django_params = {}
     django_many_to_many_params = {}
@@ -191,6 +212,7 @@ async def async_redis_dict_to_django_params(
                     django_foreign_key_instance = await update_or_create_django_instance_from_redis_instance(
                         redis_foreign_key_instance,
                         foreign_key_model,
+                        cache_conf,
                     )
                     django_params[django_field_name] = django_foreign_key_instance
                 else:
@@ -204,6 +226,7 @@ async def async_redis_dict_to_django_params(
                         django_many_to_many_instance = await update_or_create_django_instance_from_redis_instance(
                             redis_many_to_many_instance,
                             many_to_many_model,
+                            cache_conf,
                         )
                         django_many_to_many_instances.append(django_many_to_many_instance)
                 django_many_to_many_params[django_field_name] = django_many_to_many_instances
@@ -262,27 +285,6 @@ def django_sync_to_async_list(
     return result
 
 
-@sync_to_async
-def django_sync_to_async_func_and_update(
-        django_func,
-        params_to_use=None,
-        params_to_update=None,
-):
-    if type(params_to_use) == dict:
-        objects_qs = django_func(**params_to_use)
-    else:
-        objects_qs = django_func()
-
-    if type(params_to_update) == dict:
-        result = objects_qs.update(**params_to_update)
-    else:
-        result = objects_qs
-
-    result = list(result)
-
-    return result
-
-
 @sync_to_async
 def django_sync_to_async_get(
         django_instance,
@@ -292,16 +294,6 @@ def django_sync_to_async_get(
     return param
 
 
-@sync_to_async
-def django_sync_to_async_create(
-        django_model,
-        params
-):
-    new_django_instance = django_model.objects.create(**params)
-    new_django_instance_id = new_django_instance.id
-    return new_django_instance_id
-
-
 def django_to_cache(
         redis_root,
         django_model,
@@ -335,7 +327,8 @@ def django_to_cache(
                 await asyncio.gather(*async_tasks)
                 async_tasks = []
                 print(
-                    f'{datetime.datetime.now()} - Written {completed} {django_model.__name__} instances from django to cache')
+                    f'{datetime.datetime.now()} - '
+                    f'Written {completed} {django_model} instances from django to cache')
 
         if cache_conf['delete']:
             async def check_if_need_to_delete_redis_instance(redis_root, django_instances_ids, redis_instance):
@@ -352,7 +345,8 @@ def django_to_cache(
 
             await asyncio.gather(*async_tasks)
 
-        print(f'{datetime.datetime.now()} - Deleted {django_model.__name__} instances from cache')
+        print(f'{datetime.datetime.now()} - '
+              f'Deleted {django_model} instances from cache')
 
     loop = asyncio.get_event_loop()
     loop.run_until_complete(
